from __future__ import annotations

import logging
from dataclasses import dataclass
from typing import Any, Dict, List, Optional, Union

from flask import current_app

_LOGGER = logging.getLogger(__name__)


def format_users(users: Union[User, List[User]], mention: bool = False) -> str:

    if isinstance(users, User):
        users = [users]

    if mention:
        formatted_users = [user.mention() for user in users]
    else:
        formatted_users = [str(user) for user in users]

    print(formatted_users)
    return ', '.join(formatted_users)


def _get_google_chat_id(gitlab_user_id: int) -> Optional[int]:
    return current_app.config['GGCI_USER_MAPPINGS'].get(gitlab_user_id)


@dataclass
class User:
    def __init__(self, gitlab_user_id: int, name: Optional[str] = None):
        self.gitlab_user_id = gitlab_user_id
        self.name = name

    def __str__(self):
        return self.name if self.name is not None else str(self.gitlab_user_id)

    def mention(self) -> str:
        google_chat_user_id = _get_google_chat_id(self.gitlab_user_id)
        print(':::::::', google_chat_user_id)
        if google_chat_user_id is None:
            _LOGGER.warning(
                'No Google Chat user ID found for GitLab user ID %d',
                self.gitlab_user_id,
            )
            return str(self)
        return f'<users/{google_chat_user_id}>'

    @classmethod
    def from_dict(cls, user_dict: Dict[str, Any]) -> User:
        return cls(
            gitlab_user_id=user_dict['id'],
            name=user_dict.get('name'),
        )


@dataclass
class Message:
    text: str
    thread_key: Optional[str] = None



from typing import Optional

from flask import Flask

from ggci import forwarder
from ggci.config import Config, load_yaml_config


def create_app(config: Optional[Config] = None) -> Flask:

    if config is None:
        config = load_yaml_config()
        assert isinstance(config, Config)

    if not isinstance(config, Config):
        raise TypeError(
            f'config must be of type ggci.Config, got: {type(config)}'
        )

    app = Flask(__name__)
    app.register_blueprint(forwarder.bp)
    app.config.update(dict(config))
    return app



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ggci']

package_data = \
{'': ['*']}

install_requires = \
['flask>=1.1.2,<2.0.0', 'pyyaml>=5.4.1,<6.0.0', 'requests>=2.25.1,<3.0.0']

setup_kwargs = {
    'name': 'ggci',
    'version': '0.1.0',
    'description': 'GitLab Google Chat Integration',
    'long_description': None,
    'author': 'Jan Lukány',
    'author_email': 'lukany.jan@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
